# =============================================
# 
# OBJET : REALISATION D'UNE CARTE DE DISCONTINUITES DE PIB SUR LA BELGIQUE
#
# Objectif: Faire une carte de discontinuités
# auteur : Nicolas Lambert, CNRS - UMS RIATE
# version : 1.0 (oct 2014)
#
# ===============================================
rm(list=ls())

library(rgeos)
library(maptools)
library(reshape2)

# -----------------------------------------------------------
# [STEP 1] IMPORT DU FOND DE CARTE
# -----------------------------------------------------------


setwd("- votre repertoire de travail - ")
# import du fond de carte
fdc<-readShapeSpatial("geom/nuts2_2010.shp", proj4string=CRS("+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +units=m +no_defs"))
colnames(fdc@data)[1]<-"id"
plot(fdc)

# extraction d'un pays par requete : 2 premiers caracteres de l'ID = BE
head(fdc@data$id)
fdc <- fdc[substr(fdc@data$id,0,2)=="BE",]
fdc@data$id<-as.factor(as.vector(fdc@data$id))
# Affichage
plot(fdc,col="#CCCCCC")
title("Belgique\n(sans erreurs topologiques)")

# -----------------------------------------------------------
# [STEP 2] DETECTION DES REGIONS CONTIGUES
# -----------------------------------------------------------

# Création d'une matrice de contiguités (gIntersect)
contig<-gIntersects(fdc, byid = TRUE, prepared=TRUE)
row.names(contig)<-fdc@data$id
colnames(contig)<-fdc@data$id
head(contig)

# CONVERSION : Matrice -> i,j,fij (grace à reshape2)
contig<-melt(contig,variable.name=1,value.name="fij", na.rm=TRUE) 
colnames(contig)<-c("i","j","cij")
head(contig)

# Requete : On ne conserve que les couples de regions qui sont contigues
contig<-contig[contig$cij==TRUE,]
contig<-contig[contig$i!=contig$j,]
head(contig)

# On crée un id unique (concatenation des deux codes)
contig$id<-paste(contig$i,contig$j,sep = "_")
row.names(contig)<-contig$id
head(contig)

# -----------------------------------------------------------
# [STEP 3] EXTRACTION DES FRONTIERES
# -----------------------------------------------------------

# On converti les polygones et polylignes
bound<-gBoundary(fdc, byid=TRUE, id = fdc@data$id)

# On recupere la table attributiare
attribut<-fdc@data
row.names(attribut)<-attribut$id
bound<-SpatialLinesDataFrame(bound,attribut, match.ID = TRUE)
plot(bound[1], col="blue")
levels(bound@data$id)<-bound@data$id
  
# BOUCLE : pour chaque contiguité, on extrait la ligne ou eventuellement un point

nb_errors<-0
for(x in 1:nrow(contig))
{
  polyline1<-bound[bound@data$id==contig$i[x],]
  polyline2<-bound[bound@data$id==contig$j[x],]
  tmp<-gIntersection(polyline1, polyline2)
  if (class(tmp)=="SpatialLines")
  {
    tmp <- spChFIDs(tmp, as.character(contig$id[x]))
    if (exists("borders")){borders<-spRbind(borders,tmp)}
    if (!exists("borders")){borders<-tmp}  
  }

  if (class(tmp)=="SpatialPoints")
  {
  nb_errors<-nb_errors+1
  if (exists("errors")){errors<-spRbind(errors,tmp)}
  if (!exists("errors")){errors<-tmp}  
  plot(tmp,col="red",add=T)
  }
}

# Recuparation des attributs
borders<-SpatialLinesDataFrame(borders,contig, match.ID = TRUE)
borders@data<-data.frame(borders@data$id,borders@data$i,borders@data$j)
colnames(borders@data)<-c("id","i","j")


# AFfichage
plot(fdc,col="#00000050",border="white",lwd=0.1)
plot(borders,col="red",lwd=2,add=T)
head(borders@data)


# Eventuellement, on sauvagarde du shp
#writeSpatialShape(borders, "resul/BE_borders.shp", factor2char = TRUE, max_nchar=254)
#if (exists("errors")){writeSpatialShape(errors, "resul/errors.shp", factor2char = TRUE, max_nchar=254)}

# -----------------------------------------------------------
# [STEP 4] CARTOGRAPHIE
# -----------------------------------------------------------

# chargement des packages utils
library(RColorBrewer)
library(classInt)
  
# import des données  
donnees<-read.csv( "data/N2.csv",header=TRUE,sep=";",dec=",",encoding="latin1",)
donnees$var<-donnees$PIBperinh
head(donnees)

# 2. CARTE CHOROPLETHE  
head(fdc@data)
fdc@data = data.frame(fdc@data, donnees[match(fdc@data[,"id"], donnees[,"id"]),])
nbclass<-8
distr <- classIntervals(fdc@data$var,nbclass,style="quantile")$brks
colours <- brewer.pal(nbclass,"Greens")
colMap <- colours[(findInterval(fdc$var,distr,all.inside=TRUE))]
plot(fdc, col=colMap,border="black",lwd=0.1)
legend(x="topright", legend=leglabs(round(distr,1),over="plus de",under="moins de"), fill=colours, bty="n",pt.cex=0.3,cex=0.4,title="GDP/inh")

# 3. DISCONTINUITES
MaxSize<-10 # epaisseur de la plus grosse ligne
LineColor<-"red"
seuil<-1.2 # seules discontinuités relaitivse superieures à "seuil" sont retenues

# Double jointure (valeurs de part et d'autre de chaque ligne)
borders@data = data.frame(borders@data, donnees[match(borders@data[,"i"], donnees[,"id"]),])
borders@data = data.frame(borders@data, donnees[match(borders@data[,"j"], donnees[,"id"]),])

# Calcul des discontinuités (var_i / var_j)
borders@data$disc<-pmax(borders@data$var.1/borders@data$var,borders@data$var/borders@data$var.1)
borders@data<-data.frame(borders@data$id,borders@data$disc)
names(borders)<-c("id","disc")
# Tri
borders<- borders[order(borders@data$disc,decreasing=TRUE),]
# Calcul des epaisseurs
borders@data$size<-(borders@data$disc/max(na.omit((borders@data$disc))))*MaxSize 
# On ne garde que les discontinuités superieure au seuil
borders<- borders[borders@data$disc>seuil,]
# Affichage
plot(borders,col=LineColor, lwd=borders@data$size ,add=T)
# Legende
rLeg <- quantile(borders@data$size,c(0,0.5,0.80,1),type=1)
rVal <- round(quantile(borders@data$disc,c(0,0.5,0.80,1),type=1),1)
legend("bottomright",legend=rVal, lwd = rLeg, col="red",bty="n",title="Discontinuités",cex=0.4,pt.cex=1)
title(main="Discontinuités de PIB/hab en 2007",sub="Auteur: Nicolas LAMBERT, UMS RIATE, 2014",cex.sub=0.6)
